// https://github.com/umdjs/umd/blob/master/returnExports.js
(function(root, factory) {
    'use strict';

    if (typeof define === 'function' && define.amd) {
        define([], factory);
    } else if (typeof exports === 'object') {
        module.exports = factory();
    } else {
        root.microjungle = factory();
    }
}(this, function() {
    'use strict'

    var Microjungle = function(template) {
        if(!(this instanceof Microjungle)) {
            return new Microjungle(template);
        }

        this.template = template;
        this.dom = typeof window !== 'undefined';
    };

    Microjungle.prototype.createElement = function(elem, name) {
        if (this.dom) {
            return document.createElement(name);
        }

        elem += '<' + name;

        return name;
    };

    Microjungle.prototype.setAttribute = function(elem, name, val) {
        if (this.dom) {
            elem.setAttribute(name, value);
        }
        elem += ' ' + name + '="' + val + '"';
    };

    Microjungle.prototype.createTextNode = function(elem, text) {
        if (this.dom) {
            elem.appendChild(document.createTextNode(text));
        }
        elem += text;
    };

    Microjungle.prototype.monkeys = function(template, target) {
        var item;

        target = this.dom ? (target || document.createDocumentFragment()) : '';

        if (Array.isArray(template)) {
            for (var i = 0; i < template.length; i++) {
                item = template[i];

                // string or finite number
                if (typeof item === 'string' || (typeof item === 'number' && isFinite(item))) {
                    this.createTextNode(target, item);
                    // target.appendChild(document.createTextNode(item));
                    continue;
                }

                // any kind of falsy
                if (!item) {
                    continue;
                }

                // elem
                if (typeof item[0] === 'string') {
                    var elem = this.createElement(target, item.shift());
                    // var elem = document.createElement(item.shift());

                    // attrs?
                    if (Object.prototype.toString.call(item[0]) === '[object Object]') {
                        var attrs = item.shift();
                        var attrName;
                        var attrValue;

                        // attr="value"
                        for (attrName in attrs) {
                            attrValue = attrs[attrName];

                            if (
                                typeof attrValue === 'string' ||
                                (typeof attrValue === 'number' && isFinite(attrValue))
                            ) {
                                this.setAttribute(target, attrName, attrValue);
                                // elem.setAttribute(attrName, attrValue);
                            }
                        }
                    }

                    if (this.dom) {
                        target.appendChild(this.monkeys(item, elem));
                    } else {
                        target += '>';
                        // content
                        target += this.monkeys(item);
                        // closing
                        target += '</' + elem + '>';
                    }

                    // target.appendChild(monkeys(item, elem));
                    continue;
                }

                // node
                if (item.nodeType) {
                    target.appendChild(item);
                    continue;
                }

                this.monkeys(item, target);
            }
        }

        return target;
    };

    Microjungle.prototype.render = function() {
        return this.monkeys(this.template);
    };

    return Microjungle;
}));
